from .betwixtedbutterfly import BetwixtedButterfly
from .boxnovel_com import BoxNovel
from .chickengege import ChickEngege
from .chrysagarden import Chrysanthemumgarden
from .creativenovels import CreativeNovels
from .dragontea import DragonTea
from .dummynovels import DummyNovels
from .fanfiction import FanFiction
from .foxaholic import Foxaholic
from .lightnovelworld import LightNovelWorld
from .mtlnovel import MtlNovel
from .novelfull import NovelFull
from .novelfun import NovelFun
from .novelgate import NovelGate
from .novelhall import NovelHall
from .novelonlinefull import NovelOnlineFull
from .novelpassion import NovelPassion
from .novelsite import NovelSite
from .novelsrock import NovelsRock
from .peachpitting import PeachPitting
from .rainofsnow import RainOfSnow
from .readlightnovel_me import ReadLightNovel
from .readlightnovels_net import ReadLightNovelsNet
from .royalroad import RoyalRoad
from .scribblehub import ScribbleHub
from .spacebattles import Spacebattles
from .sufficientvelocity import SufficientVelocity
from .wattpad import WattPad
from .webnovel import Webnovel
from .wuxiaco import WuxiaCo
from .wuxiacom import WuxiaCom
from .wuxiaonline import WuxiaOnline
from .wuxiasite import WuxiaSite

sources = [
    BetwixtedButterfly,
    BoxNovel,
    ChickEngege,
    Chrysanthemumgarden,
    CreativeNovels,
    DragonTea,
    DummyNovels,
    FanFiction,
    Foxaholic,
    LightNovelWorld,
    MtlNovel,
    NovelFull,
    NovelFun,
    NovelGate,
    NovelHall,
    NovelOnlineFull,
    NovelPassion,
    NovelSite,
    NovelsRock,
    PeachPitting,
    RainOfSnow,
    ReadLightNovel,
    ReadLightNovelsNet,
    RoyalRoad,
    ScribbleHub,
    Spacebattles,
    SufficientVelocity,
    WattPad,
    Webnovel,
    WuxiaCo,
    WuxiaCom,
    WuxiaOnline,
    WuxiaSite,
]
